@@ -1,10 +1,11 @@
 /* SPDX-License-Identifier: LGPL-2.1+ */
 
-//#include <unistd.h>
+#include <sys/file.h>
+#include <unistd.h>
 
 //#include "alloc-util.h"
 #include "blockdev-util.h"
-//#include "btrfs-util.h"
+#include "btrfs-util.h"
 //#include "dirent-util.h"
 #include "fd-util.h"
 //#include "fileio.h"
@@ -30,6 +31,8 @@ int block_get_whole_disk(dev_t d, dev_t *ret) {
                 *ret = d;
                 return 0;
         }
+        if (errno != ENOENT)
+                return -errno;
 
         /* If it is a partition find the originating device */
         xsprintf_sys_block_path(p, "/partition", d);
@@ -59,9 +62,7 @@ int block_get_whole_disk(dev_t d, dev_t *ret) {
 int get_block_device(const char *path, dev_t *ret) {
         _cleanup_close_ int fd = -1;
         struct stat st;
-#if 0 /// elogind does not support speaking to BTRFS directly
         int r;
-#endif // 0
 
         assert(path);
         assert(ret);
@@ -81,13 +82,11 @@ int get_block_device(const char *path, dev_t *ret) {
                 return 1;
         }
 
-#if 0 /// elogind does not support speaking to BTRFS directly
         r = btrfs_get_block_device_fd(fd, ret);
         if (r > 0)
                 return 1;
         if (r != -ENOTTY) /* not btrfs */
                 return r;
-#endif // 0
 
         *ret = 0;
         return 0;
@@ -192,4 +191,31 @@ int get_block_device_harder(const char *path, dev_t *ret) {
 
         return 1;
 }
+
+int lock_whole_block_device(dev_t devt, int operation) {
+        _cleanup_free_ char *whole_node = NULL;
+        _cleanup_close_ int lock_fd = -1;
+        dev_t whole_devt;
+        int r;
+
+        /* Let's get a BSD file lock on the whole block device, as per: https://elogind.io/BLOCK_DEVICE_LOCKING */
+        /* Let's get a BSD file lock on the whole block device, as per: https://systemd.io/BLOCK_DEVICE_LOCKING */
+
+        r = block_get_whole_disk(devt, &whole_devt);
+        if (r < 0)
+                return r;
+
+        r = device_path_make_major_minor(S_IFBLK, whole_devt, &whole_node);
+        if (r < 0)
+                return r;
+
+        lock_fd = open(whole_node, O_RDONLY|O_CLOEXEC|O_NONBLOCK);
+        if (lock_fd < 0)
+                return -errno;
+
+        if (flock(lock_fd, operation) < 0)
+                return -errno;
+
+        return TAKE_FD(lock_fd);
+}
 #endif // 0
