@@ -13,11 +13,12 @@
 #include <unistd.h>
 
 #include "alloc-util.h"
-//#include "btrfs-util.h"
+#include "btrfs-util.h"
 //#include "chattr-util.h"
 #include "copy.h"
 #include "dirent-util.h"
 #include "fd-util.h"
+#include "fileio.h"
 #include "fs-util.h"
 #include "io-util.h"
 #include "macro.h"
@@ -132,7 +133,6 @@ int copy_bytes_full(
         if (ret_remains_size)
                 *ret_remains_size = 0;
 
-#if 0 /// UNNEEDED by elogind
         /* Try btrfs reflinks first. This only works on regular, seekable files, hence let's check the file offsets of
          * source and destination first. */
         if ((copy_flags & COPY_REFLINK)) {
@@ -185,7 +185,6 @@ int copy_bytes_full(
                         }
                 }
         }
-#endif // 0
 
         for (;;) {
                 ssize_t n;
@@ -575,10 +574,9 @@ static int fd_copy_directory(
         if (fdf < 0)
                 return -errno;
 
-        d = fdopendir(fdf);
+        d = take_fdopendir(&fdf);
         if (!d)
                 return -errno;
-        fdf = -1;
 
         exists = false;
         if (copy_flags & COPY_MERGE_EMPTY) {
