@@ -2,7 +2,7 @@
 #pragma once
 
 #include <stdbool.h>
-//#include <stdio.h>
+#include <stdio.h>
 
 #include "macro.h"
 
@@ -12,8 +12,9 @@ bool hostname_is_set(void);
 
 char* gethostname_malloc(void);
 #if 0 /// UNNEEDED by elogind
-int gethostname_strict(char **ret);
+char* gethostname_short_malloc(void);
 #endif // 0
+int gethostname_strict(char **ret);
 
 bool valid_ldh_char(char c) _const_;
 bool hostname_is_valid(const char *s, bool allow_trailing_dot) _pure_;
