@@ -5,14 +5,36 @@
 
 //#include <errno.h>
 #include <fcntl.h>
+#include <signal.h>
 #include <sys/syscall.h>
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <unistd.h>
 
 #ifdef ARCH_MIPS
 #include <asm/sgidefs.h>
 #endif
 
+#if defined(__x86_64__) && defined(__ILP32__)
+#define elogind_SC_arch_bias(x) ((x) | /* __X32_SYSCALL_BIT */ 0x40000000)
+#elif defined(__ia64__)
+#  define elogind_SC_arch_bias(x) (1024 + (x))
+#elif defined __alpha__
+#  define elogind_SC_arch_bias(x) (110 + (x))
+#elif defined _MIPS_SIM
+#  if _MIPS_SIM == _MIPS_SIM_ABI32
+#    define elogind_SC_arch_bias(x) (4000 + (x))
+#  elif _MIPS_SIM == _MIPS_SIM_NABI32
+#    define elogind_SC_arch_bias(x) (6000 + (x))
+#  elif _MIPS_SIM == _MIPS_SIM_ABI64
+#    define elogind_SC_arch_bias(x) (5000 + (x))
+#  else
+#    error "Unknown MIPS ABI"
+#  endif
+#else
+#define elogind_SC_arch_bias(x) (x)
+#endif
+
 #include "missing_keyctl.h"
 #include "missing_stat.h"
 
@@ -40,7 +62,7 @@ static inline int missing_pivot_root(const char *new_root, const char *put_old)
 #      undef __NR_memfd_create
 #    endif
 #    if defined __x86_64__
-#      define __NR_memfd_create 319
+#      define __NR_memfd_create elogind_SC_arch_bias(319)
 #    elif defined __arm__
 #      define __NR_memfd_create 385
 #    elif defined __aarch64__
@@ -87,7 +109,7 @@ static inline int missing_memfd_create(const char *name, unsigned int flags) {
 #      undef __NR_getrandom
 #    endif
 #    if defined __x86_64__
-#      define __NR_getrandom 318
+#      define __NR_getrandom elogind_SC_arch_bias(318)
 #    elif defined(__i386__)
 #      define __NR_getrandom 355
 #    elif defined(__arm__)
@@ -150,7 +172,7 @@ static inline pid_t missing_gettid(void) {
 #      undef __NR_name_to_handle_at
 #    endif
 #    if defined(__x86_64__)
-#      define __NR_name_to_handle_at 303
+#      define __NR_name_to_handle_at elogind_SC_arch_bias(303)
 #    elif defined(__i386__)
 #      define __NR_name_to_handle_at 341
 #    elif defined(__arm__)
@@ -159,6 +181,14 @@ static inline pid_t missing_gettid(void) {
 #      define __NR_name_to_handle_at 345
 #    elif defined(__arc__)
 #      define __NR_name_to_handle_at 264
+#    elif defined _MIPS_SIM
+#      if _MIPS_SIM == _MIPS_SIM_ABI32
+#        define elogind_NR_name_to_handle_at elogind_SC_arch_bias(339)
+#      elif _MIPS_SIM == _MIPS_SIM_NABI32
+#        define elogind_NR_name_to_handle_at elogind_SC_arch_bias(303)
+#      elif _MIPS_SIM == _MIPS_SIM_ABI64
+#        define elogind_NR_name_to_handle_at elogind_SC_arch_bias(298)
+#      endif
 #    else
 #      error "__NR_name_to_handle_at is not defined"
 #    endif
@@ -191,11 +221,19 @@ static inline int missing_name_to_handle_at(int fd, const char *name, struct fil
 #      undef __NR_setns
 #    endif
 #    if defined(__x86_64__)
-#      define __NR_setns 308
+#      define __NR_setns elogind_SC_arch_bias(308)
 #    elif defined(__i386__)
 #      define __NR_setns 346
 #    elif defined(__arc__)
 #      define __NR_setns 268
+#    elif defined _MIPS_SIM
+#      if _MIPS_SIM == _MIPS_SIM_ABI32
+#        define elogind_NR_setns elogind_SC_arch_bias(344)
+#      elif _MIPS_SIM == _MIPS_SIM_NABI32
+#        define elogind_NR_setns elogind_SC_arch_bias(308)
+#      elif _MIPS_SIM == _MIPS_SIM_ABI64
+#        define elogind_NR_setns elogind_SC_arch_bias(303)
+#      endif
 #    else
 #      error "__NR_setns is not defined"
 #    endif
@@ -232,7 +270,7 @@ static inline pid_t raw_getpid(void) {
 #      undef __NR_renameat2
 #    endif
 #    if defined __x86_64__
-#      define __NR_renameat2 316
+#      define __NR_renameat2 elogind_SC_arch_bias(316)
 #    elif defined __arm__
 #      define __NR_renameat2 382
 #    elif defined __aarch64__
@@ -333,7 +371,7 @@ static inline key_serial_t missing_request_key(const char *type, const char *des
 #      undef __NR_copy_file_range
 #    endif
 #    if defined(__x86_64__)
-#      define __NR_copy_file_range 326
+#      define __NR_copy_file_range elogind_SC_arch_bias(326)
 #    elif defined(__i386__)
 #      define __NR_copy_file_range 377
 #    elif defined __s390__
@@ -346,6 +384,14 @@ static inline key_serial_t missing_request_key(const char *type, const char *des
 #      define __NR_copy_file_range 379
 #    elif defined __arc__
 #      define __NR_copy_file_range 285
+#    elif defined _MIPS_SIM
+#      if _MIPS_SIM == _MIPS_SIM_ABI32
+#        define elogind_NR_copy_file_range elogind_SC_arch_bias(360)
+#      elif _MIPS_SIM == _MIPS_SIM_NABI32
+#        define elogind_NR_copy_file_range elogind_SC_arch_bias(324)
+#      elif _MIPS_SIM == _MIPS_SIM_ABI64
+#        define elogind_NR_copy_file_range elogind_SC_arch_bias(320)
+#      endif
 #    else
 #      warning "__NR_copy_file_range not defined for your architecture"
 #    endif
@@ -378,7 +424,7 @@ static inline ssize_t missing_copy_file_range(int fd_in, loff_t *off_in,
 #    if defined __i386__
 #      define __NR_bpf 357
 #    elif defined __x86_64__
-#      define __NR_bpf 321
+#      define __NR_bpf systemd_SC_arch_bias(321)
 #    elif defined __aarch64__
 #      define __NR_bpf 280
 #    elif defined __arm__
@@ -389,6 +435,14 @@ static inline ssize_t missing_copy_file_range(int fd_in, loff_t *off_in,
 #      define __NR_bpf 351
 #    elif defined __tilegx__
 #      define __NR_bpf 280
+#    elif defined _MIPS_SIM
+#      if _MIPS_SIM == _MIPS_SIM_ABI32
+#        define systemd_NR_bpf systemd_SC_arch_bias(355)
+#      elif _MIPS_SIM == _MIPS_SIM_NABI32
+#        define systemd_NR_bpf systemd_SC_arch_bias(319)
+#      elif _MIPS_SIM == _MIPS_SIM_ABI64
+#        define systemd_NR_bpf systemd_SC_arch_bias(315)
+#      endif
 #    else
 #      warning "__NR_bpf not defined for your architecture"
 #    endif
@@ -420,11 +474,11 @@ static inline int missing_bpf(int cmd, union bpf_attr *attr, size_t size) {
 #    if defined __i386__
 #      define __NR_pkey_mprotect 380
 #    elif defined __x86_64__
-#      define __NR_pkey_mprotect 329
+#      define __NR_pkey_mprotect elogind_SC_arch_bias(329)
 #    elif defined __arm__
 #      define __NR_pkey_mprotect 394
 #    elif defined __aarch64__
-#      define __NR_pkey_mprotect 394
+#      define __NR_pkey_mprotect 288
 #    elif defined __powerpc__
 #      define __NR_pkey_mprotect 386
 #    elif defined __s390__
@@ -453,7 +507,9 @@ static inline int missing_bpf(int cmd, union bpf_attr *attr, size_t size) {
 #    if defined __NR_statx
 #      undef __NR_statx
 #    endif
-#    if defined __aarch64__ || defined __arm__
+#    if defined __aarch64__
+#      define __NR_statx 291
+#    elif defined __arm__
 #      define __NR_statx 397
 #    elif defined __alpha__
 #      define __NR_statx 522
@@ -462,7 +518,15 @@ static inline int missing_bpf(int cmd, union bpf_attr *attr, size_t size) {
 #    elif defined __sparc__
 #      define __NR_statx 360
 #    elif defined __x86_64__
-#      define __NR_statx 332
+#      define __NR_statx elogind_SC_arch_bias(332)
+#    elif defined _MIPS_SIM
+#      if _MIPS_SIM == _MIPS_SIM_ABI32
+#        define elogind_NR_statx elogind_SC_arch_bias(366)
+#      elif _MIPS_SIM == _MIPS_SIM_NABI32
+#        define elogind_NR_statx elogind_SC_arch_bias(330)
+#      elif _MIPS_SIM == _MIPS_SIM_ABI64
+#        define elogind_NR_statx elogind_SC_arch_bias(326)
+#      endif
 #    else
 #      warning "__NR_statx not defined for your architecture"
 #    endif
@@ -526,5 +590,87 @@ static inline long missing_get_mempolicy(int *mode, unsigned long *nodemask,
         return i;
 }
 
-#define get_mempolicy missing_get_mempolicy
+#  define get_mempolicy missing_get_mempolicy
+#endif
+
+#if !HAVE_PIDFD_SEND_SIGNAL
+/* may be (invalid) negative number due to libseccomp, see PR 13319 */
+#  if ! (defined __NR_pidfd_send_signal && __NR_pidfd_send_signal >= 0)
+#    if defined __NR_pidfd_send_signal
+#      undef __NR_pidfd_send_signal
+#    endif
+/* should be always defined, see kernel 39036cd2727395c3369b1051005da74059a85317 */
+#    if defined(__alpha__)
+#      define __NR_pidfd_send_signal 534
+#    elif defined _MIPS_SIM
+#      if _MIPS_SIM == _MIPS_SIM_ABI32  /* o32 */
+#        define elogind_NR_pidfd_send_signal (424 + 4000)
+#      endif
+#      if _MIPS_SIM == _MIPS_SIM_NABI32 /* n32 */
+#        define elogind_NR_pidfd_send_signal (424 + 6000)
+#      endif
+#      if _MIPS_SIM == _MIPS_SIM_ABI64  /* n64 */
+#        define elogind_NR_pidfd_send_signal (424 + 5000)
+#      endif
+#    elif defined __ia64__
+#      define elogind_NR_pidfd_send_signal (424 + 1024)
+#    else
+#      define __NR_pidfd_send_signal elogind_SC_arch_bias(424)
+#    endif
+#  endif
+static inline int missing_pidfd_send_signal(int fd, int sig, siginfo_t *info, unsigned flags) {
+#  ifdef __NR_pidfd_send_signal
+        return syscall(__NR_pidfd_send_signal, fd, sig, info, flags);
+#  else
+        errno = ENOSYS;
+        return -1;
+#  endif
+}
+
+#  define pidfd_send_signal missing_pidfd_send_signal
+#endif
+
+#if !HAVE_PIDFD_OPEN
+/* may be (invalid) negative number due to libseccomp, see PR 13319 */
+#  if ! (defined __NR_pidfd_open && __NR_pidfd_open >= 0)
+#    if defined __NR_pidfd_open
+#      undef __NR_pidfd_open
+#    endif
+/* should be always defined, see kernel 7615d9e1780e26e0178c93c55b73309a5dc093d7 */
+#    if defined(__alpha__)
+#      define __NR_pidfd_open 544
+#    elif defined _MIPS_SIM
+#      if _MIPS_SIM == _MIPS_SIM_ABI32  /* o32 */
+#        define elogind_NR_pidfd_open (434 + 4000)
+#      endif
+#      if _MIPS_SIM == _MIPS_SIM_NABI32 /* n32 */
+#        define elogind_NR_pidfd_open (434 + 6000)
+#      endif
+#      if _MIPS_SIM == _MIPS_SIM_ABI64  /* n64 */
+#        define elogind_NR_pidfd_open (434 + 5000)
+#      endif
+#    elif defined __ia64__
+#      define elogind_NR_pidfd_open (434 + 1024)
+#    else
+#      define __NR_pidfd_open elogind_SC_arch_bias(434)
+#    endif
+#  endif
+static inline int missing_pidfd_open(pid_t pid, unsigned flags) {
+#  ifdef __NR_pidfd_open
+        return syscall(__NR_pidfd_open, pid, flags);
+#  else
+        errno = ENOSYS;
+        return -1;
+#  endif
+}
+
+#  define pidfd_open missing_pidfd_open
+#endif
+
+#if !HAVE_RT_SIGQUEUEINFO
+static inline int missing_rt_sigqueueinfo(pid_t tgid, int sig, siginfo_t *info) {
+        return syscall(__NR_rt_sigqueueinfo, tgid, sig, info);
+}
+
+#  define rt_sigqueueinfo missing_rt_sigqueueinfo
 #endif
