@@ -2,7 +2,6 @@
 
 #include <errno.h>
 #include <stdarg.h>
-#include <stdio.h>
 
 #include "macro.h"
 #include "parse-util.h"
@@ -26,7 +25,7 @@ int reset_all_signal_handlers(void) {
 
                 /* On Linux the first two RT signals are reserved by
                  * glibc, and sigaction() will return EINVAL for them. */
-                if ((sigaction(sig, &sa, NULL) < 0))
+                if (sigaction(sig, &sa, NULL) < 0)
                         if (errno != EINVAL && r >= 0)
                                 r = -errno;
         }
@@ -295,3 +294,18 @@ void nop_signal_handler(int sig) {
         /* nothing here */
 }
 #endif // 0
+
+int signal_is_blocked(int sig) {
+        sigset_t ss;
+        int r;
+
+        r = pthread_sigmask(SIG_SETMASK, NULL, &ss);
+        if (r != 0)
+                return -r;
+
+        r = sigismember(&ss, sig);
+        if (r < 0)
+                return -errno;
+
+        return r;
+}
