@@ -5,10 +5,12 @@
 
 #include "macro.h"
 
+size_t strnpcpy(char **dest, size_t size, const char *src, size_t len);
 size_t strpcpy(char **dest, size_t size, const char *src);
-#if 0 /// UNNEEDED by elogind
 size_t strpcpyf(char **dest, size_t size, const char *src, ...) _printf_(3, 4);
+#if 0 /// UNNEEDED by elogind
 size_t strpcpyl(char **dest, size_t size, const char *src, ...) _sentinel_;
 #endif // 0
+size_t strnscpy(char *dest, size_t size, const char *src, size_t len);
 size_t strscpy(char *dest, size_t size, const char *src);
 size_t strscpyl(char *dest, size_t size, const char *src, ...) _sentinel_;
