@@ -1,7 +1,6 @@
 /* SPDX-License-Identifier: LGPL-2.1+ */
 
 #include <stdlib.h>
-#include <sys/socket.h>
 
 #include "fd-util.h"
 #include "log.h"
@@ -27,9 +26,7 @@ int main(int argc, char *argv[]) {
         }
 
         elogind_set_program_name(argv[0]);
-        log_set_target(LOG_TARGET_AUTO);
-        log_parse_environment();
-        log_open();
+        log_setup_service();
 
         fd = socket(AF_UNIX, SOCK_DGRAM|SOCK_CLOEXEC, 0);
         if (fd < 0) {
