@@ -8,7 +8,6 @@
 #include "parse-printf-format.h"
 #endif // 0
 #include <stddef.h>
-//#include <sys/socket.h>
 //#include <sys/un.h>
 #include <unistd.h>
 
@@ -20,6 +19,7 @@
 #include "errno-util.h"
 #include "fd-util.h"
 #include "io-util.h"
+#include "fileio.h"
 #include "memfd-util.h"
 #include "socket-util.h"
 #include "stdio-util.h"
@@ -83,32 +83,44 @@ _public_ int sd_journal_print(int priority, const char *format, ...) {
 
 _public_ int sd_journal_printv(int priority, const char *format, va_list ap) {
 
-        /* FIXME: Instead of limiting things to LINE_MAX we could do a
-           C99 variable-length array on the stack here in a loop. */
-
 #if 0 /// elogind only needs a LINE_MAX buffer
-        char buffer[8 + LINE_MAX], p[STRLEN("PRIORITY=") + DECIMAL_STR_MAX(int) + 1];
+        char p[STRLEN("PRIORITY=") + DECIMAL_STR_MAX(int) + 1];
+        char sbuf[LINE_MAX + 8] = "MESSAGE=";
         struct iovec iov[2];
 #else // 0
-        char buffer[8 + LINE_MAX]; // We keep the +8 to not make a too big mess below.
+        char sbuf[LINE_MAX + 8] = "MESSAGE="; // We keep the +8 to not make a too big mess below.
 #endif // 0
+        int len;
+        va_list aq;
+        char *buffer = sbuf;
 
         assert_return(priority >= 0, -EINVAL);
         assert_return(priority <= 7, -EINVAL);
         assert_return(format, -EINVAL);
 
-#if 0 /// No bells and whistles needed in elogind
+#if 0 /// priority is not needed in elogind
         xsprintf(p, "PRIORITY=%i", priority & LOG_PRIMASK);
-
-        memcpy(buffer, "MESSAGE=", 8);
 #endif // 0
-        vsnprintf(buffer+8, sizeof(buffer) - 8, format, ap);
+
+        va_copy(aq, ap);
+        len = vsnprintf(buffer + 8, LINE_MAX, format, aq);
+        va_end(aq);
+
+        if (len >= (int)LONG_LINE_MAX - 8)
+                return -ENOBUFS;
+
+        /* Allocate large buffer to accommodate big message */
+        if (len >= LINE_MAX) {
+                buffer = alloca(len + 9);
+                memcpy(buffer, "MESSAGE=", 8);
+                assert_se(vsnprintf(buffer + 8, len + 1, format, ap) == len);
+        }
 
         /* Strip trailing whitespace, keep prefix whitespace. */
         (void) strstrip(buffer);
 
         /* Suppress empty lines */
-        if (isempty(buffer+8))
+        if (isempty(buffer + 8))
                 return 0;
 
 #if 0 /// As elogind does not talk to systemd-journal, use syslog.
@@ -515,9 +527,13 @@ _public_ int sd_journal_print_with_location(int priority, const char *file, cons
 
 _public_ int sd_journal_printv_with_location(int priority, const char *file, const char *line, const char *func, const char *format, va_list ap) {
 #if 0 /// UNNEEDED by elogind
-        char buffer[8 + LINE_MAX], p[STRLEN("PRIORITY=") + DECIMAL_STR_MAX(int) + 1];
+        char p[STRLEN("PRIORITY=") + DECIMAL_STR_MAX(int) + 1];
+        char sbuf[LINE_MAX + 8] = "MESSAGE=";
         struct iovec iov[5];
         char *f;
+        int len;
+        char *buffer = sbuf;
+        va_list aq;
 #endif // 0
 
         assert_return(priority >= 0, -EINVAL);
@@ -527,14 +543,25 @@ _public_ int sd_journal_printv_with_location(int priority, const char *file, con
 #if 0 /// As elogind sends to syslog anyway, take a shortcut here
         xsprintf(p, "PRIORITY=%i", priority & LOG_PRIMASK);
 
-        memcpy(buffer, "MESSAGE=", 8);
-        vsnprintf(buffer+8, sizeof(buffer) - 8, format, ap);
+        va_copy(aq, ap);
+        len = vsnprintf(buffer + 8, LINE_MAX, format, aq);
+        va_end(aq);
+
+        if (len >= (int)LONG_LINE_MAX - 8)
+                return -ENOBUFS;
+
+        /* Allocate large buffer to accommodate big message */
+        if (len >= LINE_MAX) {
+                buffer = alloca(len + 9);
+                memcpy(buffer, "MESSAGE=", 8);
+                assert_se(vsnprintf(buffer + 8, len + 1, format, ap) == len);
+        }
 
         /* Strip trailing whitespace, keep prefixing whitespace */
         (void) strstrip(buffer);
 
         /* Suppress empty lines */
-        if (isempty(buffer+8))
+        if (isempty(buffer + 8))
                 return 0;
 
         /* func is initialized from __func__ which is not a macro, but
