@@ -1,13 +1,9 @@
 /* SPDX-License-Identifier: LGPL-2.1+ */
 
 #include <fcntl.h>
-#include <pwd.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <linux/vt.h>
-#if ENABLE_UTMP
-#include <utmpx.h>
-#endif
 
 #include "sd-device.h"
 
@@ -17,6 +13,7 @@
 #include "cgroup-util.h"
 #include "conf-parser.h"
 #include "device-util.h"
+#include "efi-loader.h"
 #include "errno-util.h"
 #include "fd-util.h"
 #include "limits-util.h"
@@ -28,8 +25,11 @@
 #include "terminal-util.h"
 #include "udev-util.h"
 #include "user-util.h"
+#include "userdb.h"
 /// Additional includes needed by elogind
 #include "elogind.h"
+#include "sleep-config.h"
+#include "utmp-wtmp.h"
 
 void manager_reset_config(Manager *m) {
         assert(m);
@@ -63,7 +63,7 @@ void manager_reset_config(Manager *m) {
         m->idle_action = HANDLE_IGNORE;
 
         m->runtime_dir_size = physical_memory_scale(10U, 100U); /* 10% */
-        m->user_tasks_max = system_tasks_max_scale(DEFAULT_USER_TASKS_MAX_PERCENTAGE, 100U); /* 33% */
+        m->runtime_dir_inodes = DIV_ROUND_UP(m->runtime_dir_size, 4096); /* 4k per inode */
         m->sessions_max = 8192;
         m->inhibitors_max = 8192;
 
@@ -80,72 +80,68 @@ void elogind_manager_reset_config(Manager* m) {
         int dbg_cnt;
 #endif // ENABLE_DEBUG_ELOGIND
 
-        /* Set default Sleep config if not already set by logind.conf */
-        if (!m->suspend_state)
-                m->suspend_state = strv_new("mem", "standby", "freeze", NULL);
-        if (!m->hibernate_mode)
-                m->hibernate_mode = strv_new("platform", "shutdown", NULL);
-        if (!m->hibernate_state)
-                m->hibernate_state = strv_new("disk", NULL);
-        if (!m->hybrid_sleep_mode)
-                m->hybrid_sleep_mode = strv_new("suspend", "platform", "shutdown", NULL);
-        if (!m->hybrid_sleep_state)
-                m->hybrid_sleep_state = strv_new("disk", NULL);
-        if (!m->hibernate_delay_sec)
-                m->hibernate_delay_sec = 180 * USEC_PER_MINUTE;
+        (void)parse_sleep_config(&m);
 
 #if ENABLE_DEBUG_ELOGIND
         dbg_cnt = -1;
-        while (m->suspend_mode && m->suspend_mode[++dbg_cnt])
-                log_debug_elogind("suspend_mode[%d] = %s",
-                                  dbg_cnt, m->suspend_mode[dbg_cnt]);
+        while (m->suspend_modes && m->suspend_modes[++dbg_cnt])
+                log_debug_elogind("suspend_modes[%d] = %s",
+                                  dbg_cnt, m->suspend_modes[dbg_cnt]);
         dbg_cnt = -1;
-        while (m->suspend_state[++dbg_cnt])
-                log_debug_elogind("suspend_state[%d] = %s",
-                                  dbg_cnt, m->suspend_state[dbg_cnt]);
+        while (m->suspend_states && m->suspend_states[++dbg_cnt])
+                log_debug_elogind("suspend_states[%d] = %s",
+                                  dbg_cnt, m->suspend_states[dbg_cnt]);
         dbg_cnt = -1;
-        while (m->hibernate_mode[++dbg_cnt])
-                log_debug_elogind("hibernate_mode[%d] = %s",
-                                  dbg_cnt, m->hibernate_mode[dbg_cnt]);
+        while (m->hibernate_modes && m->hibernate_modes[++dbg_cnt])
+                log_debug_elogind("hibernate_modes[%d] = %s",
+                                  dbg_cnt, m->hibernate_modes[dbg_cnt]);
         dbg_cnt = -1;
-        while (m->hibernate_state[++dbg_cnt])
-                log_debug_elogind("hibernate_state[%d] = %s",
-                                  dbg_cnt, m->hibernate_state[dbg_cnt]);
+        while (m->hibernate_states && m->hibernate_states[++dbg_cnt])
+                log_debug_elogind("hibernate_states[%d] = %s",
+                                  dbg_cnt, m->hibernate_states[dbg_cnt]);
         dbg_cnt = -1;
-        while (m->hybrid_sleep_mode[++dbg_cnt])
-                log_debug_elogind("hybrid_sleep_mode[%d] = %s",
-                                  dbg_cnt, m->hybrid_sleep_mode[dbg_cnt]);
+        while (m->hybrid_modes && m->hybrid_modes[++dbg_cnt])
+                log_debug_elogind("hybrid_modes[%d] = %s",
+                                  dbg_cnt, m->hybrid_modes[dbg_cnt]);
         dbg_cnt = -1;
-        while (m->hybrid_sleep_state[++dbg_cnt])
-                log_debug_elogind("hybrid_sleep_state[%d] = %s",
-                                  dbg_cnt, m->hybrid_sleep_state[dbg_cnt]);
+        while (m->hybrid_states && m->hybrid_states[++dbg_cnt])
+                log_debug_elogind("hybrid_states[%d] = %s",
+                                  dbg_cnt, m->hybrid_states[dbg_cnt]);
         log_debug_elogind("hibernate_delay_sec: %lu seconds (%lu minutes)",
                           m->hibernate_delay_sec / USEC_PER_SEC,
                           m->hibernate_delay_sec / USEC_PER_MINUTE);
 #endif // ENABLE_DEBUG_ELOGIND
 }
 #endif // 1
+
 int manager_parse_config_file(Manager *m) {
         assert(m);
 
 #if 0 /// elogind parses its own config file
-        return config_parse_many_nulstr(PKGSYSCONFDIR "/logind.conf",
-                                        CONF_PATHS_NULSTR("systemd/logind.conf.d"),
-                                        "Login\0",
-                                        config_item_perf_lookup, logind_gperf_lookup,
+        return config_parse_many_nulstr(
+                        PKGSYSCONFDIR "/logind.conf",
+                        CONF_PATHS_NULSTR("systemd/logind.conf.d"),
+                        "Login\0",
+                        config_item_perf_lookup, logind_gperf_lookup,
+                        CONFIG_PARSE_WARN, m,
+                        NULL);
 #else // 0
         const char* logind_conf = getenv("ELOGIND_CONF_FILE");
 
         if (!logind_conf)
                 logind_conf = PKGSYSCONFDIR "/logind.conf";
 
-        return config_parse(NULL, logind_conf, NULL, "Login\0Sleep\0",
-                            config_item_perf_lookup, logind_gperf_lookup,
+        return config_parse_many_nulstr(
+                        logind_conf,
+                        CONF_PATHS_NULSTR("elogind/logind.conf.d"),
+                        "Login\0Sleep\0",
+                        config_item_perf_lookup, logind_gperf_lookup,
+                        CONFIG_PARSE_WARN, m,
+                        NULL);
 #endif // 0
-                                        CONFIG_PARSE_WARN, m);
 }
 
-int manager_add_device(Manager *m, const char *sysfs, bool master, Device **_device) {
+int manager_add_device(Manager *m, const char *sysfs, bool master, Device **ret_device) {
         Device *d;
 
         assert(m);
@@ -161,13 +157,13 @@ int manager_add_device(Manager *m, const char *sysfs, bool master, Device **_dev
                         return -ENOMEM;
         }
 
-        if (_device)
-                *_device = d;
+        if (ret_device)
+                *ret_device = d;
 
         return 0;
 }
 
-int manager_add_seat(Manager *m, const char *id, Seat **_seat) {
+int manager_add_seat(Manager *m, const char *id, Seat **ret_seat) {
         Seat *s;
         int r;
 
@@ -181,13 +177,13 @@ int manager_add_seat(Manager *m, const char *id, Seat **_seat) {
                         return r;
         }
 
-        if (_seat)
-                *_seat = s;
+        if (ret_seat)
+                *ret_seat = s;
 
         return 0;
 }
 
-int manager_add_session(Manager *m, const char *id, Session **_session) {
+int manager_add_session(Manager *m, const char *id, Session **ret_session) {
         Session *s;
         int r;
 
@@ -201,35 +197,32 @@ int manager_add_session(Manager *m, const char *id, Session **_session) {
                         return r;
         }
 
-        if (_session)
-                *_session = s;
+        if (ret_session)
+                *ret_session = s;
 
         return 0;
 }
 
 int manager_add_user(
                 Manager *m,
-                uid_t uid,
-                gid_t gid,
-                const char *name,
-                const char *home,
-                User **_user) {
+                UserRecord *ur,
+                User **ret_user) {
 
         User *u;
         int r;
 
         assert(m);
-        assert(name);
+        assert(ur);
 
-        u = hashmap_get(m->users, UID_TO_PTR(uid));
+        u = hashmap_get(m->users, UID_TO_PTR(ur->uid));
         if (!u) {
-                r = user_new(&u, m, uid, gid, name, home);
+                r = user_new(&u, m, ur);
                 if (r < 0)
                         return r;
         }
 
-        if (_user)
-                *_user = u;
+        if (ret_user)
+                *ret_user = u;
 
         return 0;
 }
@@ -237,34 +230,37 @@ int manager_add_user(
 int manager_add_user_by_name(
                 Manager *m,
                 const char *name,
-                User **_user) {
+                User **ret_user) {
 
-        const char *home = NULL;
-        uid_t uid;
-        gid_t gid;
+        _cleanup_(user_record_unrefp) UserRecord *ur = NULL;
         int r;
 
         assert(m);
         assert(name);
 
-        r = get_user_creds(&name, &uid, &gid, &home, NULL, 0);
+        r = userdb_by_name(name, USERDB_AVOID_SHADOW, &ur);
         if (r < 0)
                 return r;
 
-        return manager_add_user(m, uid, gid, name, home, _user);
+        return manager_add_user(m, ur, ret_user);
 }
 
-int manager_add_user_by_uid(Manager *m, uid_t uid, User **_user) {
-        struct passwd *p;
+int manager_add_user_by_uid(
+                Manager *m,
+                uid_t uid,
+                User **ret_user) {
+
+        _cleanup_(user_record_unrefp) UserRecord *ur = NULL;
+        int r;
 
         assert(m);
+        assert(uid_is_valid(uid));
 
-        errno = 0;
-        p = getpwuid(uid);
-        if (!p)
-                return errno_or_else(ENOENT);
+        r = userdb_by_uid(uid, USERDB_AVOID_SHADOW, &ur);
+        if (r < 0)
+                return r;
 
-        return manager_add_user(m, uid, p->pw_gid, p->pw_name, p->pw_dir, _user);
+        return manager_add_user(m, ur, ret_user);
 }
 
 int manager_add_inhibitor(Manager *m, const char* id, Inhibitor **ret) {
@@ -287,7 +283,7 @@ int manager_add_inhibitor(Manager *m, const char* id, Inhibitor **ret) {
         return 0;
 }
 
-int manager_add_button(Manager *m, const char *name, Button **_button) {
+int manager_add_button(Manager *m, const char *name, Button **ret_button) {
         Button *b;
 
         assert(m);
@@ -300,8 +296,8 @@ int manager_add_button(Manager *m, const char *name, Button **_button) {
                         return -ENOMEM;
         }
 
-        if (_button)
-                *_button = b;
+        if (ret_button)
+                *ret_button = b;
 
         return 0;
 }
@@ -429,44 +425,33 @@ int manager_get_session_by_pid(Manager *m, pid_t pid, Session **ret) {
         if (!s) {
 #if 0 /// elogind does not support systemd units, but its own session system
                 r = cg_pid_get_unit(pid, &unit);
-                if (r < 0)
-                        goto not_found;
-
-                s = hashmap_get(m->session_units, unit);
-                if (!s)
-                        goto not_found;
+                if (r >= 0)
+                        s = hashmap_get(m->session_units, unit);
 #else // 0
                 log_debug_elogind("Searching session for PID %u", pid);
                 r = cg_pid_get_session(pid, &session_name);
-                if (r < 0)
-                        goto not_found;
 
-                s = hashmap_get(m->sessions, session_name);
-                if (!s)
-                        goto not_found;
+                if (r >= 0)
+                        s = hashmap_get(m->sessions, session_name);
+
                 log_debug_elogind("Session Name \"%s\" -> Session \"%s\"",
-                                  session_name, s && s->id ? s->id : "NULL");
+                                  strnull(session_name), s && s->id ? s->id : "(null)");
 #endif // 0
         }
 
         if (ret)
                 *ret = s;
 
-        return 1;
-
-not_found:
-        if (ret)
-                *ret = NULL;
-        return 0;
+        return !!s;
 }
 
 int manager_get_user_by_pid(Manager *m, pid_t pid, User **ret) {
 #if 0 /// elogind does not support systemd units, but its own session system
         _cleanup_free_ char *unit = NULL;
-        User *u;
 #else // 0
         Session *s;
 #endif // 0
+        User *u = NULL;
         int r;
 
         assert(m);
@@ -476,32 +461,19 @@ int manager_get_user_by_pid(Manager *m, pid_t pid, User **ret) {
 
 #if 0 /// elogind does not support systemd units, but its own session system
         r = cg_pid_get_slice(pid, &unit);
-        if (r < 0)
-                goto not_found;
-
-        u = hashmap_get(m->user_units, unit);
-        if (!u)
-                goto not_found;
+        if (r >= 0)
+                u = hashmap_get(m->user_units, unit);
+#else // 0
+        // If a session was found, ignore it if it is already closing.
+        r = manager_get_session_by_pid (m, pid, &s);
+        if ( (r > 0) && (SESSION_CLOSING != session_get_state(s)) )
+                u = s->user;
+#endif // 0
 
         if (ret)
                 *ret = u;
 
-#else // 0
-        // If a session was found, ignore it if it is already closing.
-        r = manager_get_session_by_pid (m, pid, &s);
-        if (r <= 0 || SESSION_CLOSING == session_get_state(s))
-                goto not_found;
-
-        if (ret)
-                *ret =  s->user;
-#endif // 0
-        return 1;
-
-not_found:
-        if (ret)
-                *ret = NULL;
-
-        return 0;
+        return !!u;
 }
 
 int manager_get_idle_hint(Manager *m, dual_timestamp *t) {
@@ -720,10 +692,10 @@ static int manager_count_external_displays(Manager *m) {
                 if (sd_device_get_sysname(d, &nn) < 0)
                         continue;
 
-                /* Ignore internal displays: the type is encoded in the sysfs name, as the second dash separated item
-                 * (the first is the card name, the last the connector number). We implement a blacklist of external
-                 * displays here, rather than a whitelist of internal ones, to ensure we don't block suspends too
-                 * eagerly. */
+                /* Ignore internal displays: the type is encoded in the sysfs name, as the second dash
+                 * separated item (the first is the card name, the last the connector number). We implement a
+                 * deny list of external displays here, rather than an allow list of internal ones, to ensure
+                 * we don't block suspends too eagerly. */
                 dash = strchr(nn, '-');
                 if (!dash)
                         continue;
@@ -804,13 +776,14 @@ bool manager_all_buttons_ignored(Manager *m) {
 int manager_read_utmp(Manager *m) {
 #if ENABLE_UTMP
         int r;
+        _cleanup_(utxent_cleanup) bool utmpx = false;
 
         assert(m);
 
         if (utmpxname(_PATH_UTMPX) < 0)
                 return log_error_errno(errno, "Failed to set utmp path to " _PATH_UTMPX ": %m");
 
-        setutxent();
+        utmpx = utxent_start();
 
         for (;;) {
                 _cleanup_free_ char *t = NULL;
@@ -823,8 +796,7 @@ int manager_read_utmp(Manager *m) {
                 if (!u) {
                         if (errno != 0)
                                 log_warning_errno(errno, "Failed to read " _PATH_UTMPX ", ignoring: %m");
-                        r = 0;
-                        break;
+                        return 0;
                 }
 
                 if (u->ut_type != USER_PROCESS)
@@ -834,18 +806,14 @@ int manager_read_utmp(Manager *m) {
                         continue;
 
                 t = strndup(u->ut_line, sizeof(u->ut_line));
-                if (!t) {
-                        r = log_oom();
-                        break;
-                }
+                if (!t)
+                        return log_oom();
 
                 c = path_startswith(t, "/dev/");
                 if (c) {
                         r = free_and_strdup(&t, c);
-                        if (r < 0) {
-                                log_oom();
-                                break;
-                        }
+                        if (r < 0)
+                                return log_oom();
                 }
 
                 if (isempty(t))
@@ -875,8 +843,6 @@ int manager_read_utmp(Manager *m) {
                 log_debug("Acquired TTY information '%s' from utmp for session '%s'.", s->tty, s->id);
         }
 
-        endutxent();
-        return r;
 #else
         return 0;
 #endif
@@ -939,3 +905,27 @@ void manager_reconnect_utmp(Manager *m) {
         manager_connect_utmp(m);
 #endif
 }
+
+int manager_read_efi_boot_loader_entries(Manager *m) {
+#if ENABLE_EFI
+        int r;
+
+        assert(m);
+        if (m->efi_boot_loader_entries_set)
+                return 0;
+
+        r = efi_loader_get_entries(&m->efi_boot_loader_entries);
+        if (r == -ENOENT || ERRNO_IS_NOT_SUPPORTED(r)) {
+                log_debug_errno(r, "Boot loader reported no entries.");
+                m->efi_boot_loader_entries_set = true;
+                return 0;
+        }
+        if (r < 0)
+                return log_error_errno(r, "Failed to determine entries reported by boot loader: %m");
+
+        m->efi_boot_loader_entries_set = true;
+        return 1;
+#else
+        return 0;
+#endif
+}
