@@ -477,9 +477,8 @@ int user_start(User *u) {
         if (r < 0)
                 return r;
 #endif // 1
-
         /* Save the user data so far, because pam_elogind will read the XDG_RUNTIME_DIR out of it while starting up
-         * elogind --user.  We need to do user_save_internal() because we have not "officially" started yet. */
+         * systemd --user.  We need to do user_save_internal() because we have not "officially" started yet. */
         user_save_internal(u);
 
         /* Set slice parameters */
@@ -1031,9 +1030,11 @@ int config_parse_compat_user_tasks_max(
         log_syntax(unit, LOG_NOTICE, filename, line, 0,
                    "Support for option %s= has been removed.",
                    lvalue);
-        log_info("Hint: try creating /etc/elogind/system/user-.slice.d/50-limits.conf with:\n"
+#if 0 /// elogind does not use slices
+        log_info("Hint: try creating /etc/systemd/system/user-.slice.d/50-limits.conf with:\n"
                  "        [Slice]\n"
                  "        TasksMax=%s",
                  rvalue);
+#endif // 0
         return 0;
 }
