@@ -2,39 +2,43 @@
 
 #include <errno.h>
 #include <fcntl.h>
-#include <string.h>
+#include <sys/types.h>
 #include <unistd.h>
 
 #include "sd-daemon.h"
 #include "sd-device.h"
 
 #include "alloc-util.h"
-#include "bus-error.h"
+//#include "bus-error.h"
+#include "bus-locator.h"
+#include "bus-log-control-api.h"
 #include "bus-polkit.h"
-#include "cgroup-util.h"
-#include "def.h"
+//#include "cgroup-util.h"
+#include "daemon-util.h"
+//#include "def.h"
 #include "device-util.h"
 #include "dirent-util.h"
 #include "fd-util.h"
 #include "format-util.h"
 #include "fs-util.h"
 #include "logind-dbus.h"
-#include "logind-seat-dbus.h"
-#include "logind-session-dbus.h"
-#include "logind-user-dbus.h"
+//#include "logind-seat-dbus.h"
+//#include "logind-session-dbus.h"
+//#include "logind-user-dbus.h"
 #include "logind.h"
 #include "main-func.h"
 #include "parse-util.h"
 #include "process-util.h"
 #include "selinux-util.h"
+#include "service-util.h"
 #include "signal-util.h"
 #include "strv.h"
 #include "terminal-util.h"
 /// Additional includes needed by elogind
-#include "cgroup.h"       // From src/core/
 #include "label.h"
 #include "musl_missing.h"
 #include "udev-util.h"
+#include "user-util.h"
 
 static Manager* manager_unref(Manager *m);
 DEFINE_TRIVIAL_CLEANUP_FUNC(Manager*, manager_unref);
@@ -191,6 +195,9 @@ static Manager* manager_unref(Manager *m) {
         free(m->action_job);
 #endif // 0
 
+        strv_free(m->efi_boot_loader_entries);
+        free(m->efi_loader_entry_one_shot);
+
         return mfree(m);
 }
 
@@ -353,17 +360,24 @@ static int manager_enumerate_users(Manager *m) {
 
         FOREACH_DIRENT(de, d, return -errno) {
                 User *u;
+                uid_t uid;
 
                 if (!dirent_is_file(de))
                         continue;
 
-                k = manager_add_user_by_name(m, de->d_name, &u);
+                k = parse_uid(de->d_name, &uid);
+                if (k < 0) {
+                        r = log_warning_errno(k, "Failed to parse filename /run/systemd/users/%s as UID.", de->d_name);
+                        continue;
+                }
+
+                k = manager_add_user_by_uid(m, uid, &u);
                 if (k < 0) {
                         r = log_warning_errno(k, "Failed to add user by file name %s, ignoring: %m", de->d_name);
                         continue;
                 }
 
-                log_debug_elogind("Loading user %d \"%s\"", u->uid, u->name ? u->name : "n/a");
+                log_debug_elogind("Loading user %d \"%s\"", u->user_record->uid, strna(u->user_record->user_name) );
                 user_add_to_gc_queue(u);
 
                 k = user_load(u);
@@ -660,54 +674,20 @@ static int manager_connect_bus(Manager *m) {
         if (r < 0)
                 return log_error_errno(r, "Failed to connect to system bus: %m");
 
-        r = sd_bus_add_object_vtable(m->bus, NULL, "/org/freedesktop/login1", "org.freedesktop.login1.Manager", manager_vtable, m);
+        r = bus_add_implementation(m->bus, &manager_object, m);
         if (r < 0)
-                return log_error_errno(r, "Failed to add manager object vtable: %m");
+                return r;
 
-        r = sd_bus_add_fallback_vtable(m->bus, NULL, "/org/freedesktop/login1/seat", "org.freedesktop.login1.Seat", seat_vtable, seat_object_find, m);
+        r = bus_log_control_api_register(m->bus);
         if (r < 0)
-                return log_error_errno(r, "Failed to add seat object vtable: %m");
-
-        r = sd_bus_add_node_enumerator(m->bus, NULL, "/org/freedesktop/login1/seat", seat_node_enumerator, m);
-        if (r < 0)
-                return log_error_errno(r, "Failed to add seat enumerator: %m");
-
-        r = sd_bus_add_fallback_vtable(m->bus, NULL, "/org/freedesktop/login1/session", "org.freedesktop.login1.Session", session_vtable, session_object_find, m);
-        if (r < 0)
-                return log_error_errno(r, "Failed to add session object vtable: %m");
-
-        r = sd_bus_add_node_enumerator(m->bus, NULL, "/org/freedesktop/login1/session", session_node_enumerator, m);
-        if (r < 0)
-                return log_error_errno(r, "Failed to add session enumerator: %m");
-
-        r = sd_bus_add_fallback_vtable(m->bus, NULL, "/org/freedesktop/login1/user", "org.freedesktop.login1.User", user_vtable, user_object_find, m);
-        if (r < 0)
-                return log_error_errno(r, "Failed to add user object vtable: %m");
-
-        r = sd_bus_add_node_enumerator(m->bus, NULL, "/org/freedesktop/login1/user", user_node_enumerator, m);
-        if (r < 0)
-                return log_error_errno(r, "Failed to add user enumerator: %m");
+                return r;
 
 #if 0 /// elogind does not support systemd as PID 1
-        r = sd_bus_match_signal_async(
-                        m->bus,
-                        NULL,
-                        "org.freedesktop.systemd1",
-                        "/org/freedesktop/systemd1",
-                        "org.freedesktop.systemd1.Manager",
-                        "JobRemoved",
-                        match_job_removed, NULL, m);
+        r = bus_match_signal_async(m->bus, NULL, bus_systemd_mgr, "JobRemoved", match_job_removed, NULL, m);
         if (r < 0)
                 return log_error_errno(r, "Failed to request match for JobRemoved: %m");
 
-        r = sd_bus_match_signal_async(
-                        m->bus,
-                        NULL,
-                        "org.freedesktop.systemd1",
-                        "/org/freedesktop/systemd1",
-                        "org.freedesktop.systemd1.Manager",
-                        "UnitRemoved",
-                        match_unit_removed, NULL, m);
+        r = bus_match_signal_async(m->bus, NULL, bus_systemd_mgr, "UnitRemoved", match_unit_removed, NULL, m);
         if (r < 0)
                 return log_error_errno(r, "Failed to request match for UnitRemoved: %m");
 
@@ -722,26 +702,11 @@ static int manager_connect_bus(Manager *m) {
         if (r < 0)
                 return log_error_errno(r, "Failed to request match for PropertiesChanged: %m");
 
-        r = sd_bus_match_signal_async(
-                        m->bus,
-                        NULL,
-                        "org.freedesktop.systemd1",
-                        "/org/freedesktop/systemd1",
-                        "org.freedesktop.systemd1.Manager",
-                        "Reloading",
-                        match_reloading, NULL, m);
+        r = bus_match_signal_async(m->bus, NULL, bus_systemd_mgr, "Reloading", match_reloading, NULL, m);
         if (r < 0)
                 return log_error_errno(r, "Failed to request match for Reloading: %m");
 
-        r = sd_bus_call_method_async(
-                        m->bus,
-                        NULL,
-                        "org.freedesktop.systemd1",
-                        "/org/freedesktop/systemd1",
-                        "org.freedesktop.systemd1.Manager",
-                        "Subscribe",
-                        NULL, NULL,
-                        NULL);
+        r = bus_call_method_async(m->bus, NULL, bus_systemd_mgr, "Subscribe", NULL, NULL, NULL);
         if (r < 0)
                 return log_error_errno(r, "Failed to enable subscription: %m");
 #endif // 0
@@ -1249,38 +1214,52 @@ static int manager_run(Manager *m) {
 
 static int run(int argc, char *argv[]) {
         _cleanup_(manager_unrefp) Manager *m = NULL;
+        _cleanup_(notify_on_cleanup) const char *notify_message = NULL;
         int r;
 
-#if 1 /// perform extra checks for elogind startup
-        r = elogind_startup(argc, argv);
-        if (r)
-                return r < 0 ? EXIT_FAILURE : EXIT_SUCCESS;
-#endif // 1
         elogind_set_program_name(argv[0]);
         log_set_facility(LOG_AUTH);
-#if ENABLE_DEBUG_ELOGIND
-        log_set_max_level(LOG_DEBUG);
-        log_set_target(LOG_TARGET_SYSLOG_OR_KMSG);
-#endif // ENABLE_DEBUG_ELOGIND
         log_setup_service();
 
-        umask(0022);
+#if 1 /// perform extra checks for elogind startup, and fork if wanted
+        bool has_forked = false;
+        r = elogind_startup(argc, argv, &has_forked);
+        log_debug_elogind("elogind_startup() exited with %d", r);
+        if (r)
+                return r < 0 ? EXIT_FAILURE : EXIT_SUCCESS;
+        // If we forked, we are in the grandchild, the daemon, now.
+#endif // 1
 
-#if 0 /// elogind has some extra functionality at startup, argc can be != 1
-        if (argc != 1) {
-                log_error("This program takes no arguments.");
-                return -EINVAL;
+#if 0 /// This is elogind
+        r = service_parse_argv("systemd-logind.service",
+                               "Manager for user logins and devices and privileged operations.",
+                               BUS_IMPLEMENTATIONS(&manager_object,
+                                                   &log_control_object),
+                               argc, argv);
+        if (r <= 0)
+                return r;
+#else // 0
+        if (!has_forked) {
+                r = service_parse_argv("elogind",
+                                       "Manager for user logins and devices and privileged operations.",
+                                       BUS_IMPLEMENTATIONS(&manager_object, &log_control_object),
+                                       argc, argv);
+                log_debug_elogind("service_parse_argv() returned %d", r);
+                if (r <= 0)
+                        return r;
         }
 #endif // 0
 
+        umask(0022);
+
         r = mac_selinux_init();
         if (r < 0)
-                return log_error_errno(r, "Could not initialize labelling: %m");
+                return r;
 
-#if 0 /// elogind can not rely on systemd to help, so we need a bit more effort than this
         /* Always create the directories people can create inotify watches in. Note that some applications might check
          * for the existence of /run/systemd/seats/ to determine whether logind is available, so please always make
          * sure these directories are created early on and unconditionally. */
+#if 0 /// elogind can not rely on systemd to help, so we need a bit more effort than this
         (void) mkdir_label("/run/systemd/seats", 0755);
         (void) mkdir_label("/run/systemd/users", 0755);
         (void) mkdir_label("/run/systemd/sessions", 0755);
@@ -1303,12 +1282,12 @@ static int run(int argc, char *argv[]) {
 #endif // 0
 
 #if 0 /// elogind also blocks SIGQUIT, and installs a signal handler for it
-
         assert_se(sigprocmask_many(SIG_BLOCK, NULL, SIGHUP, SIGTERM, SIGINT, SIGCHLD, -1) >= 0);
 #else // 0
-        assert_se(sigprocmask_many(SIG_BLOCK, NULL, SIGHUP, SIGTERM, SIGINT, SIGQUIT, -1) >= 0);
+        assert_se(sigprocmask_many(SIG_BLOCK, NULL, SIGHUP, SIGTERM, SIGINT, SIGCHLD, SIGQUIT, -1) >= 0);
 #endif // 0
 
+        log_debug_elogind("%s", "Creating manager...");
         r = manager_new(&m);
         if (r < 0)
                 return log_error_errno(r, "Failed to allocate manager object: %m");
@@ -1318,31 +1297,23 @@ static int run(int argc, char *argv[]) {
 #if 1 /// elogind needs an Add-On for sleep configuration
         elogind_manager_reset_config(m);
 #endif // 1
+        log_debug_elogind("%s", "Starting manager...");
         r = manager_startup(m);
         if (r < 0)
                 return log_error_errno(r, "Failed to fully start up daemon: %m");
 
-        log_debug("elogind running as pid "PID_FMT, getpid_cached());
-        (void) sd_notify(false,
-                         "READY=1\n"
-                         "STATUS=Processing requests...");
-
-        r = manager_run(m);
 
 #if 1 /// Do not tell anybody if elogind is restarted through an INTerrupt
         if (m->do_interrupt) {
                 log_debug_elogind("elogind interrupted (PID "PID_FMT"), going down silently...", getpid_cached());
         } else {
 #endif /// 1
-        log_debug("elogind stopped as pid "PID_FMT, getpid_cached());
-        (void) sd_notify(false,
-                         "STOPPING=1\n"
-                         "STATUS=Shutting down...");
+        notify_message = notify_start(NOTIFY_READY, NOTIFY_STOPPING);
 #if 1 /// extra bracket for elogind here
         }
 #endif // 1
 
-        return r;
+        return manager_run(m);
 }
 
 DEFINE_MAIN_FUNCTION(run);
