@@ -7,4 +7,4 @@
 int dev_setup(const char *prefix, uid_t uid, gid_t gid);
 #endif // 0
 
-int make_inaccessible_nodes(const char *root, uid_t uid, gid_t gid);
+int make_inaccessible_nodes(const char *parent_dir, uid_t uid, gid_t gid);
