@@ -2,19 +2,19 @@
 
 #include <errno.h>
 
+#include "errno-util.h"
 #include "format-util.h"
 #include "log.h"
 #include "procfs-util.h"
+#include "tests.h"
 
 int main(int argc, char *argv[]) {
 #if 0 /// elogind only needs v
         char buf[CONST_MAX(FORMAT_TIMESPAN_MAX, FORMAT_BYTES_MAX)];
         nsec_t nsec;
+#endif // 0
         uint64_t v;
         int r;
-#else // 0
-        uint64_t v;
-#endif // 0
 
         log_parse_environment();
         log_open();
@@ -30,7 +30,11 @@ int main(int argc, char *argv[]) {
         log_info("Current number of tasks: %" PRIu64, v);
 #endif // 0
 
-        assert_se(procfs_tasks_get_limit(&v) >= 0);
+        r = procfs_tasks_get_limit(&v);
+        if (r == -ENOENT || ERRNO_IS_PRIVILEGE(r))
+                return log_tests_skipped("can't read /proc/sys/kernel/pid_max");
+
+        assert_se(r >= 0);
         log_info("Limit of tasks: %" PRIu64, v);
         assert_se(v > 0);
 #if 0 /// UNSUPPORTED by elogind (we aren't init)
