@@ -151,7 +151,7 @@ static void test_device_path_make_canonical(void) {
         test_device_path_make_canonical_one("/dev/tty");
 
 #if 0 /// UNNEEDED by elogind
-        if (is_device_node("/run/systemd/inaccessible/chr") > 0) {
+        if (is_device_node("/run/systemd/inaccessible/blk") > 0) {
                 test_device_path_make_canonical_one("/run/systemd/inaccessible/chr");
                 test_device_path_make_canonical_one("/run/systemd/inaccessible/blk");
         }
